/****************************************************************************
** Meta object code from reading C++ file 'dbusscreensaver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/dbusscreensaver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusscreensaver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DBusScreenSaver_t {
    QByteArrayData data[27];
    char stringdata0[486];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DBusScreenSaver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DBusScreenSaver_t qt_meta_stringdata_DBusScreenSaver = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DBusScreenSaver"
QT_MOC_LITERAL(1, 16, 21), // "allScreenSaverChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 14), // "allScreenSaver"
QT_MOC_LITERAL(4, 54, 32), // "batteryScreenSaverTimeoutChanged"
QT_MOC_LITERAL(5, 87, 25), // "batteryScreenSaverTimeout"
QT_MOC_LITERAL(6, 113, 34), // "linePowerScreenSaverTimeoutCh..."
QT_MOC_LITERAL(7, 148, 27), // "linePowerScreenSaverTimeout"
QT_MOC_LITERAL(8, 176, 25), // "currentScreenSaverChanged"
QT_MOC_LITERAL(9, 202, 18), // "currentScreenSaver"
QT_MOC_LITERAL(10, 221, 16), // "isRunningChanged"
QT_MOC_LITERAL(11, 238, 9), // "isRunning"
QT_MOC_LITERAL(12, 248, 24), // "lockScreenAtAwakeChanged"
QT_MOC_LITERAL(13, 273, 17), // "lockScreenAtAwake"
QT_MOC_LITERAL(14, 291, 22), // "lockScreenDelayChanged"
QT_MOC_LITERAL(15, 314, 15), // "lockScreenDelay"
QT_MOC_LITERAL(16, 330, 24), // "onSessionPropertyChanged"
QT_MOC_LITERAL(17, 355, 9), // "interface"
QT_MOC_LITERAL(18, 365, 18), // "changed_properties"
QT_MOC_LITERAL(19, 384, 12), // "QDBusMessage"
QT_MOC_LITERAL(20, 397, 7), // "message"
QT_MOC_LITERAL(21, 405, 21), // "onDBusPropertyChanged"
QT_MOC_LITERAL(22, 427, 15), // "onConfigChanged"
QT_MOC_LITERAL(23, 443, 3), // "key"
QT_MOC_LITERAL(24, 447, 20), // "onInputEventReceived"
QT_MOC_LITERAL(25, 468, 12), // "QEvent::Type"
QT_MOC_LITERAL(26, 481, 4) // "type"

    },
    "DBusScreenSaver\0allScreenSaverChanged\0"
    "\0allScreenSaver\0batteryScreenSaverTimeoutChanged\0"
    "batteryScreenSaverTimeout\0"
    "linePowerScreenSaverTimeoutChanged\0"
    "linePowerScreenSaverTimeout\0"
    "currentScreenSaverChanged\0currentScreenSaver\0"
    "isRunningChanged\0isRunning\0"
    "lockScreenAtAwakeChanged\0lockScreenAtAwake\0"
    "lockScreenDelayChanged\0lockScreenDelay\0"
    "onSessionPropertyChanged\0interface\0"
    "changed_properties\0QDBusMessage\0message\0"
    "onDBusPropertyChanged\0onConfigChanged\0"
    "key\0onInputEventReceived\0QEvent::Type\0"
    "type"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DBusScreenSaver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       7,  110, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       4,    1,   72,    2, 0x06 /* Public */,
       6,    1,   75,    2, 0x06 /* Public */,
       8,    1,   78,    2, 0x06 /* Public */,
      10,    1,   81,    2, 0x06 /* Public */,
      12,    1,   84,    2, 0x06 /* Public */,
      14,    1,   87,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    3,   90,    2, 0x08 /* Private */,
      21,    3,   97,    2, 0x08 /* Private */,
      22,    1,  104,    2, 0x08 /* Private */,
      24,    1,  107,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void, QMetaType::Int,   15,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, 0x80000000 | 19,   17,   18,   20,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, 0x80000000 | 19,   17,   18,   20,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void, 0x80000000 | 25,   26,

 // properties: name, type, flags
       3, QMetaType::QStringList, 0x00495001,
       5, QMetaType::Int, 0x00495103,
       7, QMetaType::Int, 0x00495103,
       9, QMetaType::QString, 0x00495103,
      11, QMetaType::Bool, 0x00495001,
      13, QMetaType::Bool, 0x00495103,
      15, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,

       0        // eod
};

void DBusScreenSaver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DBusScreenSaver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->allScreenSaverChanged((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 1: _t->batteryScreenSaverTimeoutChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->linePowerScreenSaverTimeoutChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->currentScreenSaverChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->isRunningChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->lockScreenAtAwakeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->lockScreenDelayChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onSessionPropertyChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QDBusMessage(*)>(_a[3]))); break;
        case 8: _t->onDBusPropertyChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QDBusMessage(*)>(_a[3]))); break;
        case 9: _t->onConfigChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onInputEventReceived((*reinterpret_cast< QEvent::Type(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DBusScreenSaver::*)(QStringList );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::allScreenSaverChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DBusScreenSaver::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::batteryScreenSaverTimeoutChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DBusScreenSaver::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::linePowerScreenSaverTimeoutChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DBusScreenSaver::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::currentScreenSaverChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DBusScreenSaver::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::isRunningChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (DBusScreenSaver::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::lockScreenAtAwakeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (DBusScreenSaver::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DBusScreenSaver::lockScreenDelayChanged)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DBusScreenSaver *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QStringList*>(_v) = _t->allScreenSaver(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->batteryScreenSaverTimeout(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->linePowerScreenSaverTimeout(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->currentScreenSaver(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isRunning(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->lockScreenAtAwake(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->lockScreenDelay(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DBusScreenSaver *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setBatteryScreenSaverTimeout(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setLinePowerScreenSaverTimeout(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setCurrentScreenSaver(*reinterpret_cast< QString*>(_v)); break;
        case 5: _t->setLockScreenAtAwake(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setLockScreenDelay(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DBusScreenSaver::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DBusScreenSaver.data,
    qt_meta_data_DBusScreenSaver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DBusScreenSaver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DBusScreenSaver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DBusScreenSaver.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DBusScreenSaver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DBusScreenSaver::allScreenSaverChanged(QStringList _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DBusScreenSaver::batteryScreenSaverTimeoutChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DBusScreenSaver::linePowerScreenSaverTimeoutChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DBusScreenSaver::currentScreenSaverChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void DBusScreenSaver::isRunningChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void DBusScreenSaver::lockScreenAtAwakeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void DBusScreenSaver::lockScreenDelayChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
