/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a screensaveradaptor -i /build/deepin-screensaver/src/deepin-screensaver/src/dbusscreensaver.h -l DBusScreenSaver /build/deepin-screensaver/src/deepin-screensaver/src/com.deepin.ScreenSaver.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "screensaveradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ScreenSaverAdaptor
 */

ScreenSaverAdaptor::ScreenSaverAdaptor(DBusScreenSaver *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ScreenSaverAdaptor::~ScreenSaverAdaptor()
{
    // destructor
}

QStringList ScreenSaverAdaptor::allScreenSaver() const
{
    // get the value of property allScreenSaver
    return qvariant_cast< QStringList >(parent()->property("allScreenSaver"));
}

int ScreenSaverAdaptor::batteryScreenSaverTimeout() const
{
    // get the value of property batteryScreenSaverTimeout
    return qvariant_cast< int >(parent()->property("batteryScreenSaverTimeout"));
}

void ScreenSaverAdaptor::setBatteryScreenSaverTimeout(int value)
{
    // set the value of property batteryScreenSaverTimeout
    parent()->setProperty("batteryScreenSaverTimeout", QVariant::fromValue(value));
}

QString ScreenSaverAdaptor::currentScreenSaver() const
{
    // get the value of property currentScreenSaver
    return qvariant_cast< QString >(parent()->property("currentScreenSaver"));
}

void ScreenSaverAdaptor::setCurrentScreenSaver(const QString &value)
{
    // set the value of property currentScreenSaver
    parent()->setProperty("currentScreenSaver", QVariant::fromValue(value));
}

bool ScreenSaverAdaptor::isRunning() const
{
    // get the value of property isRunning
    return qvariant_cast< bool >(parent()->property("isRunning"));
}

int ScreenSaverAdaptor::linePowerScreenSaverTimeout() const
{
    // get the value of property linePowerScreenSaverTimeout
    return qvariant_cast< int >(parent()->property("linePowerScreenSaverTimeout"));
}

void ScreenSaverAdaptor::setLinePowerScreenSaverTimeout(int value)
{
    // set the value of property linePowerScreenSaverTimeout
    parent()->setProperty("linePowerScreenSaverTimeout", QVariant::fromValue(value));
}

bool ScreenSaverAdaptor::lockScreenAtAwake() const
{
    // get the value of property lockScreenAtAwake
    return qvariant_cast< bool >(parent()->property("lockScreenAtAwake"));
}

void ScreenSaverAdaptor::setLockScreenAtAwake(bool value)
{
    // set the value of property lockScreenAtAwake
    parent()->setProperty("lockScreenAtAwake", QVariant::fromValue(value));
}

int ScreenSaverAdaptor::lockScreenDelay() const
{
    // get the value of property lockScreenDelay
    return qvariant_cast< int >(parent()->property("lockScreenDelay"));
}

void ScreenSaverAdaptor::setLockScreenDelay(int value)
{
    // set the value of property lockScreenDelay
    parent()->setProperty("lockScreenDelay", QVariant::fromValue(value));
}

QStringList ScreenSaverAdaptor::ConfigurableItems()
{
    // handle method call com.deepin.ScreenSaver.ConfigurableItems
    return parent()->ConfigurableItems();
}

QString ScreenSaverAdaptor::GetScreenSaverCover(const QString &name)
{
    // handle method call com.deepin.ScreenSaver.GetScreenSaverCover
    return parent()->GetScreenSaverCover(name);
}

bool ScreenSaverAdaptor::IsConfigurable(const QString &name)
{
    // handle method call com.deepin.ScreenSaver.IsConfigurable
    return parent()->IsConfigurable(name);
}

bool ScreenSaverAdaptor::Preview(const QString &name, int staysOn)
{
    // handle method call com.deepin.ScreenSaver.Preview
    return parent()->Preview(name, staysOn);
}

void ScreenSaverAdaptor::RefreshScreenSaverList()
{
    // handle method call com.deepin.ScreenSaver.RefreshScreenSaverList
    parent()->RefreshScreenSaverList();
}

void ScreenSaverAdaptor::Start()
{
    // handle method call com.deepin.ScreenSaver.Start
    parent()->Start();
}

bool ScreenSaverAdaptor::StartCustomConfig(const QString &name)
{
    // handle method call com.deepin.ScreenSaver.StartCustomConfig
    return parent()->StartCustomConfig(name);
}

void ScreenSaverAdaptor::Stop()
{
    // handle method call com.deepin.ScreenSaver.Stop
    parent()->Stop();
}


#include "screensaveradaptor.moc"
